package easik.database.types;

//~--- non-JDK imports --------------------------------------------------------

//~--- JDK imports ------------------------------------------------------------
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

import easik.xml.xsd.nodes.types.XSDBaseType;
import easik.xml.xsd.nodes.types.XSDType;

/**
 * A class representing a SMALLINT SQL data type.  A SMALLINT can contain values
 * from -2^15 to 2^15-1.  Some databases may not support SMALLINT and may be
 * automatically converted to an INTEGER instead.  See also Integer and BigInt.
 */
public class SmallInt extends EasikType
{
    /**
     *
     *
     * @return
     */
    public String toString()
    {
        return "SMALLINT";
    }

    /**
     *
     *
     * @param input
     *
     * @return
     */
    public boolean verifyInput(final String input)
    {
        return input.matches("^[-+]?\\d{1,5}$");
    }

    /**
     *
     *
     * @return
     */
    public int getSqlType()
    {
        return Types.SMALLINT;
    }

    /**
     *
     *
     * @param ps
     * @param col
     * @param value
     *
     * @throws SQLException
     */
    public void bindValue(final PreparedStatement ps, final int col, final String value) throws SQLException
    {
        ps.setInt(col, Integer.parseInt(value));
    }

    /**
     *
     *
     * @return
     */
    public XSDType getXMLSchemaType()
    {
        return XSDBaseType.xsShort;
    }
}
