import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import easik.overview.Overview;
import easik.overview.vertex.SketchNode;
import easik.overview.vertex.ViewNode;
import easik.sketch.Sketch;
import easik.sketch.edge.NormalEdge;
import easik.sketch.edge.SketchEdge;
import easik.sketch.vertex.EntityNode;
import easik.ui.ApplicationFrame;
import easik.ui.SketchFrame;
import easik.ui.ViewFrame;
import easik.view.View;
import easik.view.util.QueryException;
import easik.view.vertex.QueryNode;


public class CreatingAndDeleting {
	//overview stuff
	static Overview _overview;
	static ApplicationFrame _AppFrame;
	
	//sketch stuff
	static Sketch _sketch;
	static SketchNode _sketchNode;
	static SketchFrame _sketchFrame;
	
	//sketch
	static View _view;
	static ViewFrame _viewFrame;
	static ViewNode _viewNode;
	
	
	@BeforeClass
	public static void testSetup() throws QueryException {
		 _AppFrame = new ApplicationFrame();
		_overview = _AppFrame.getOverview();
		
		_sketchNode = _overview.addNewSketch("sketch", 20, 20);
		_sketchFrame = _sketchNode.getFrame();
		_sketch = _sketchNode.getMModel();
		_sketch.setWarnings(false);
		
		_viewNode = _overview.addNewView(_sketchNode, "view");
		_viewFrame = _viewNode.getFrame();
		_view = _viewNode.getMModel();
		
		EntityNode en1 = new EntityNode("A", _sketch);
		EntityNode en2 = new EntityNode("B", _sketch);
		_sketch.addEntity(en1,en2);
		
		NormalEdge sk = new NormalEdge(en1, en2, "edge");
		_sketch.addEdge(sk);
		
		QueryNode qn1 = new QueryNode("v_A", _view, "Select * from A");
		QueryNode qn2 = new QueryNode("v_B", _view, "Select * from B");
		
		_view.addEntity(qn1, qn2);
	}

	@AfterClass
	public static void testCleanup() {
		 _AppFrame = null;
		_overview = null;
		
		_sketchFrame = null;
		_sketchNode = null;
		_sketch = null;
		
		_viewFrame = null;
		_viewNode = null;
		_view = null;
	}
	
	@Test
	public void testDeleteEverything() throws QueryException{
		/*
		 * Deleting View and components
		 */
		ArrayList<QueryNode> entities = new ArrayList<QueryNode>(_view.getEntities());
		for(QueryNode qn : entities){
			_view.removeNode(qn);	
		}
		assertEquals("View should have no queryNodes", 0, _view.getEntities().size());
		
		assertEquals("View should have no edges", 0, _view.getEdges().size());
		
		_overview.removeView(_viewNode);

		assertEquals("Sketch should have no view", 0, _sketch.getViews().size());
		
		/*
		 * Deleting Sketch and components
		 */
		ArrayList<SketchEdge> edges = new ArrayList<SketchEdge>(_sketch.getEdges().values());
		for(SketchEdge e : edges){
			_sketch.removeEdge(e);;	
		}
		assertEquals("Sketch should have no edges", 0, _sketch.getEdges().size());
		
		ArrayList<EntityNode> ent = new ArrayList<EntityNode>(_sketch.getEntities());
		for(EntityNode qn : ent){
			_sketch.removeNode(qn);	
		}
		assertEquals("Sketch should have no queryNodes", 0, _view.getEntities().size());
		
		_overview.removeSketch(_sketchNode);
		assertEquals("Overview should have 0 sketches", 0, _overview.getSketches().size());
	}

}
