import static org.junit.Assert.assertEquals;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import easik.overview.Overview;
import easik.overview.vertex.SketchNode;
import easik.sketch.Sketch;
import easik.sketch.edge.NormalEdge;
import easik.sketch.edge.PartialEdge;
import easik.sketch.vertex.EntityNode;
import easik.ui.ApplicationFrame;
import easik.ui.SketchFrame;
import easik.view.util.QueryException;


public class CyclesTest {
	//overview stuff
		static Overview _overview;
		static ApplicationFrame _AppFrame;
		
		//sketch stuff
		static Sketch _sketch;
		static SketchNode _sketchNode;
		static SketchFrame _sketchFrame;
		
		//sketch stuff
		static Sketch _sketch1;
		static SketchNode _sketchNode1;
		static SketchFrame _sketchFrame1;
		
		//sketch stuff
		static Sketch _sketch2;
		static SketchNode _sketchNode2;
		static SketchFrame _sketchFrame2;
		
		//sketch stuff
		static Sketch _sketch3;
		static SketchNode _sketchNode3;
		static SketchFrame _sketchFrame3;
		
		@BeforeClass
		public static void testSetup() {
			 _AppFrame = new ApplicationFrame();
			_overview = _AppFrame.getOverview();
			
			_sketchNode = _overview.addNewSketch("sketch", 20, 20);
			_sketchFrame = _sketchNode.getFrame();
			_sketch = _sketchNode.getMModel();
			_sketch.setWarnings(false);
			
			_sketchNode1 = _overview.addNewSketch("sketch1", 20, 20);
			_sketchFrame1 = _sketchNode1.getFrame();
			_sketch1 = _sketchNode1.getMModel();
			_sketch1.setWarnings(false);
			
			_sketchNode2 = _overview.addNewSketch("sketch2", 20, 20);
			_sketchFrame2 = _sketchNode2.getFrame();
			_sketch2 = _sketchNode2.getMModel();
			_sketch2.setWarnings(false);
			
			_sketchNode3 = _overview.addNewSketch("sketch3", 20, 20);
			_sketchFrame3 = _sketchNode3.getFrame();
			_sketch3 = _sketchNode3.getMModel();
			_sketch3.setWarnings(false);
		}

		@AfterClass
		public static void testCleanup() {
			 _AppFrame = null;
			_overview = null;
			
			_sketchFrame = null;
			_sketchNode = null;
			_sketch = null;
			
			_sketchFrame1 = null;
			_sketchNode1 = null;
			_sketch1 = null;
			
			_sketchFrame2 = null;
			_sketchNode2 = null;
			_sketch2 = null;
		}

		/*
		 * Basic cycle where two nodes point to eachother
		 */
		@Test
		public void testCycles() throws QueryException{
			EntityNode en1 = new EntityNode("A", _sketch);
			EntityNode en2 = new EntityNode("B", _sketch);
			_sketch.addEntity(en1,en2);
			
			NormalEdge sk = new NormalEdge(en1, en2, "e1");
			_sketch.addEdge(sk);
			assertEquals("There should be no cycle", false, _sketch.getHasCycle());
			
			NormalEdge sk1 = new NormalEdge(en2, en1, "e2");
			_sketch.addEdge(sk1);
			assertEquals("There should be a cycle", true, _sketch.getHasCycle());
		}
		
		
		/*
		 * Nullable cycle, shouldn't give warning
		 * then same cycle but not nullable should give warning
		 */
		@Test
		public void testCycles1() throws QueryException{
			EntityNode en1 = new EntityNode("A", _sketch1);
			EntityNode en2 = new EntityNode("B", _sketch1);
			EntityNode en3 = new EntityNode("C", _sketch1);
			_sketch1.addEntity(en1,en2, en3);
			
			NormalEdge sk = new NormalEdge(en1, en2, "e1");
			_sketch1.addEdge(sk);
			assertEquals("There should be no cycle", false, _sketch1.getHasCycle());
			
			NormalEdge sk1 = new NormalEdge(en2, en3, "e2");
			_sketch1.addEdge(sk1);
			assertEquals("There should be no cycle", false, _sketch1.getHasCycle());
			
			PartialEdge sk2 = new PartialEdge(en3, en1, "e3p");
			_sketch1.addEdge(sk2);
			assertEquals("There should be no cycle", false, _sketch1.getHasCycle());
			
			NormalEdge sk3 = new NormalEdge(en3, en1, "e3");
			_sketch1.addEdge(sk3);
			assertEquals("There should be a cycle", true, _sketch1.getHasCycle());
		}
		
		/*
		 * multiple nodes going to same node, shouldn't be cycle
		 */
		@Test
		public void testCycles2() throws QueryException{
			assertEquals("There should be no cycle", false, _sketch2.getHasCycle());

			EntityNode en1 = new EntityNode("A", _sketch2);
			EntityNode en2 = new EntityNode("B", _sketch2);
			_sketch2.addEntity(en1,en2);
			
			NormalEdge sk = new NormalEdge(en1, en2, "e1");
			_sketch2.addEdge(sk);
			assertEquals("There should be no cycle", false, _sketch2.getHasCycle());
			
			NormalEdge sk1 = new NormalEdge(en1, en2, "e2");
			_sketch2.addEdge(sk1);
			assertEquals("There should be no cycle", false, _sketch2.getHasCycle());
			
			PartialEdge sk2 = new PartialEdge(en1, en2, "e3p");
			_sketch2.addEdge(sk2);
			assertEquals("There should be no cycle", false, _sketch2.getHasCycle());
			
			NormalEdge sk3 = new NormalEdge(en2, en1, "e3");
			_sketch2.addEdge(sk3);
			assertEquals("There should be a cycle", true, _sketch2.getHasCycle());
		}
		
		/*
		 * Tests a diagram as seen below with all arrows pointing down.
		 * Should not be a cycle.
		 * 
		 * 				x
		 * 			/		\
		 * 		x				x
		 * 			\		/	
		 * 				x
		 */
		@Test
		public void testCycles3() throws QueryException{
			EntityNode en1 = new EntityNode("A", _sketch3);
			EntityNode en2 = new EntityNode("B", _sketch3);
			EntityNode en3 = new EntityNode("C", _sketch3);
			EntityNode en4 = new EntityNode("D", _sketch3);
			_sketch3.addEntity(en1,en2,en3,en4);
			
			NormalEdge sk = new NormalEdge(en1, en2, "e1");
			_sketch3.addEdge(sk);
			assertEquals("There should be no cycle", false, _sketch3.getHasCycle());
			
			NormalEdge sk1 = new NormalEdge(en1, en3, "e2");
			_sketch3.addEdge(sk1);
			assertEquals("There should be no cycle", false, _sketch3.getHasCycle());
			
			NormalEdge sk2 = new NormalEdge(en2, en4, "e3p");
			_sketch3.addEdge(sk2);
			assertEquals("There should be no cycle", false, _sketch3.getHasCycle());
			
			NormalEdge sk3 = new NormalEdge(en3, en4, "e3");
			_sketch3.addEdge(sk3);
			assertEquals("There should be no cycle", false, _sketch3.getHasCycle());
		}
		
		
		
}
