import static org.junit.Assert.*;

import java.io.File;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import easik.overview.Overview;
import easik.overview.vertex.SketchNode;
import easik.overview.vertex.ViewNode;
import easik.sketch.Sketch;
import easik.sketch.edge.NormalEdge;
import easik.sketch.vertex.EntityNode;
import easik.ui.ApplicationFrame;
import easik.ui.SketchFrame;
import easik.ui.ViewFrame;
import easik.view.View;
import easik.view.util.QueryException;
import easik.view.vertex.QueryNode;

public class exportingAndLoading {
		//overview stuff
		static Overview _overview;
		static ApplicationFrame _AppFrame;
		
		//sketch stuff
		static Sketch _sketch;
		static SketchNode _sketchNode;
		static SketchFrame _sketchFrame;
		
		//sketch
		static View _view;
		static ViewFrame _viewFrame;
		static ViewNode _viewNode;
		
		@BeforeClass
		public static void testSetup() {
			 _AppFrame = new ApplicationFrame();
			_overview = _AppFrame.getOverview();
				
			_sketchNode = _overview.addNewSketch("sketch", 20, 20);
			_sketchFrame = _sketchNode.getFrame();
			_sketch = _sketchNode.getMModel();
			_sketch.setWarnings(false);
			
			_viewNode = _overview.addNewView(_sketchNode, "view");
			_viewFrame = _viewNode.getFrame();
			_view = _viewNode.getMModel();
		}

		@AfterClass
		public static void testCleanup() {
			 _AppFrame = null;
			_overview = null;
			
			_sketchFrame = null;
			_sketchNode = null;
			_sketch = null;
			
			_viewFrame = null;
			_viewNode = null;
			_view = null;
		}
		
		/*
		 * Adds two entities to sketch and an edge between them
		 * queries both and adds to view then exports the overview.
		 */
		@Test
		public void testExportAndLoadWithView() throws QueryException{
			EntityNode en1 = new EntityNode("A", _sketch);
			EntityNode en2 = new EntityNode("B", _sketch);
			_sketch.addEntity(en1,en2);
			
			NormalEdge sk = new NormalEdge(en1, en2, "e");
			_sketch.addEdge(sk);
			
			QueryNode qn1 = new QueryNode("v_A", _view, "Select * from A");
			QueryNode qn2 = new QueryNode("v_B", _view, "Select * from B");
			_view.addEntity(qn1, qn2);
			
	        assertEquals("check for entityNode A", "A", _sketch.getEntity("A").getName());
	        assertEquals("check for entityNode B", "B", _sketch.getEntity("B").getName());
	        assertEquals("check for queryNode v_A", "v_A", _view.getEntity("v_A").getName());
	        assertEquals("check for queryNode v_B", "v_B", _view.getEntity("v_B").getName());
	        assertEquals("check sketch for edge", true, _sketch.getEdges().containsKey("e"));
			assertEquals("check view for edge", true, _view.getEdges().containsKey("e"));
			
	        File selFile = new File("C:\\Users\\Max Sigurdson\\Documents\\GitHub\\EASIK\\sketches\\junitTests.easik");
	        
	        _overview.getDocInfo().updateModificationDate();
	        _overview.saveToXML(selFile);
	        _overview.setFile(selFile);
	        _overview.setDirty(false);
	        
	        File otherFile = new File("C:\\Users\\Max Sigurdson\\Documents\\GitHub\\EASIK\\sketches\\sample.easik");
	        _overview.openOverview(otherFile);
	        
	        _overview.openOverview(selFile);
	        assertEquals("check for entityNode A", "A", _sketch.getEntity("A").getName());
	        assertEquals("check for entityNode B", "B", _sketch.getEntity("B").getName());
	        assertEquals("check for queryNode v_A", "v_A", _view.getEntity("v_A").getName());
	        assertEquals("check for queryNode v_B", "v_B", _view.getEntity("v_B").getName());
	        assertEquals("check sketch for edge", true, _sketch.getEdges().containsKey("e"));
			assertEquals("check view for edge", true, _view.getEdges().containsKey("e"));
	        
	        
	        
			
		}
}
